#include <delays.h>
#include <pwm.h>

/*
C4 - F = 262 Hz, T = 3.8167 mSec, T/2 = 1.9083
D4 - F = 294 Hz, T = 3.4013 mSec, T/2 = 1.7006
E4 - F = 330 Hz, T = 3.0303 mSec, T/2 = 1.5151
F4 - F = 349 Hz, T = 2.8653 mSec, T/2 = 1.4326
G4 - F = 392 Hz, T = 2.5510 mSec, T/2 = 1.2755
A4 - F = 440 Hz, T = 2.2727 mSec, T/2 = 1.13635
B4 - F = 494 Hz, T = 2.0246 mSec, T/2 = 1.0123
C4 - F = 524 Hz, T = 1.9110 mSec, T/2 = 0.9555
*/

#include "main.h"
#pragma code

void playNote( const unsigned char ucNote, const unsigned char ucDelay )
{
	// play note
	OpenPWM1( g_pucPeriod[ ucNote ] );
	SetDCPWM1( 100 );

	// pause
	Delay10KTCYx( ucDelay );
}

void noSound( const unsigned char ucDelay )
{
	// stop PWM
	ClosePWM1();

	// pause
	Delay10KTCYx( ucDelay );
}

void main()
{
	int iMelodyLen;

	// select frequency - 110 : 4Mhz
	OSCCONbits.IRCF2 = 1;
	OSCCONbits.IRCF1 = 1;
	OSCCONbits.IRCF0 = 0;

	// select internal oscilator
	OSCCONbits.SCS1 = 1;
	OSCCONbits.SCS0 = 0;

	// setup timer prescaler - 1:16 (MAX) needed by the PWM module
	T2CONbits.T2CKPS1 = 1;
	T2CONbits.T2CKPS0 = 0;

	// start timer 2
	T2CONbits.TMR2ON = 1;

	// configure PORTC and PORTB as output
	TRISC = 0X00;
	TRISB = 0X00;

	// configure PORTD
	TRISDbits.TRISD0 = 1;
	TRISDbits.TRISD1 = 1;
	TRISDbits.TRISD2 = 0;
	TRISDbits.TRISD3 = 0;

	PORTDbits.RD2 = 0;
	PORTDbits.RD3 = 0;

	PORTB = 0;

	iMelodyLen = sizeof( g_pucMelody1 );
	while( 1 )
	{
		int i;
		for ( i = 0 ; i < iMelodyLen ; ++i )
		{
			if ( ( PORTDbits.RD0 == 0 ) || ( PORTDbits.RD1 == 0 ) )
			{
				playNote( g_pucMelody1[ i ], g_pucNoteLength1[ i ] / 2 );
			}
			else
			{
				playNote( g_pucMelody1[ i ], g_pucNoteLength1[ i ] );
			}
			noSound( INNER_DELAY );

			// change LEDs state
			PORTDbits.RD2 = !PORTDbits.RD2;
			PORTDbits.RD3 = !PORTDbits.RD3;

			PORTBbits.RB0 = !PORTBbits.RB0;
			PORTBbits.RB1 = !PORTBbits.RB1;
			PORTBbits.RB2 = !PORTBbits.RB2;
		}

		Delay10KTCYx( MELODY_END );
	}
}
